#!/usr/bin/env python3
# vim: set ts=4 sw=4 et sts=4 ai:

"""
Functions needed by hdmi2usb-mode-switch implemented using libusb.

This should be portable to Mac, Windows and Linux.
"""

import logging
import usb
if usb.__file__.endswith('.so'):
    logging.warning("Your python usb module is old.")
import usb.util

from .base import *


class LibDevice(DeviceBase):

    def inuse(self, dev=None):
        try:
            if dev is None:
                dev = usb.core.find(bus=self.path.bus,
                                    address=self.path.address)

            # config = dev.get_active_configuration()
            active = False
            for config in dev:
                for i, inf in enumerate(config):
                    inf_active = dev.is_kernel_driver_active(
                        inf.bInterfaceNumber)
                    active = active or inf_active
            return active
        except usb.core.USBError:
            return None

    def detach(self):
        # Detach any driver currently attached.
        dev = usb.core.find(bus=self.path.bus, address=self.path.address)

        if not self.inuse(dev):
            return True

        config = dev.get_active_configuration()
        for inf in config:
            if dev.is_kernel_driver_active(inf.bInterfaceNumber):
                dev.detach_kernel_driver(inf.bInterfaceNumber)


Device = LibDevice


def find_usb_devices():
    busses = usb.busses()

    devobjs = []
    for dev in usb.core.find(find_all=True):
        serialno = None
        if dev.iSerialNumber > 0:
            try:
                serialno = dev.serial_number
            except usb.USBError:
                pass
            except ValueError:
                pass

        did = None
        try:
            did = '%04x' % dev.bcdDevice
        except TypeError:
            pass

        devobjs.append(
            LibDevice(
                vid=dev.idVendor,
                pid=dev.idProduct,
                did=did,
                serialno=serialno,
                path=Path(
                    bus=dev.bus,
                    address=dev.address)))
    return devobjs



#!/usr/bin/env python
# vim: set ts=4 sw=4 et sts=4 ai:

import array
import usb.core
import usb.util


VC_EEPROM = 0xB1
READ_EEPROM = 0xC0
WRITE_EEPROM = 0x40


def get_eeprom(dev, addr, amount):
    data = array.array('B')
    while len(data) < amount:
        transfer_size = min(64, amount - len(data))

        result = dev.ctrl_transfer(
            READ_EEPROM,
            VC_EEPROM,
            addr + len(data),
            0,
            transfer_size,
        )
        assert len(result) == transfer_size, "len(result) %i == %i" % (
            len(result), transfer_size)

        data += result

    return data


def set_eeprom(dev, addr, data):
    offset = 0
    while offset < len(data):
        transfer_size = min(32, len(data) - offset)
        result = dev.ctrl_transfer(
            WRITE_EEPROM,
            VC_EEPROM,
            addr + offset,
            0,
            data[offset:offset + transfer_size],
        )
        assert result == transfer_size, "result %i == %i" % (
            result, transfer_size)
        offset += transfer_size


def get_dev():
    # find our device
    dev = usb.core.find(idVendor=0x2A19, idProduct=0x5441)

    # was it found?
    if dev is None:
        raise ValueError('Device not found')

    dev.set_configuration()
    return dev


if __name__ == "__main__":
    import sys
    import argparse
    import time

    dev = get_dev()
    current_eeprom_data = get_eeprom(dev, 0, 256)

    old_eeprom_data = bytes(current_eeprom_data)
    print(repr(old_eeprom_data))

    if len(sys.argv) > 1:
        new_eeprom_data = file(sys.argv[1], "rb").read()
        print(repr(old_eeprom_data))
        print(repr(new_eeprom_data))

        if old_eeprom_data != new_eeprom_data:
            set_eeprom(dev, 0, new_eeprom_data)
    else:
        print(repr(s))
        s.check()



#!/usr/bin/env python
# vim: set ts=4 sw=4 et sts=4 ai:

"""
Tool to figure find the USB device that a board is available at.

This is a complicated process as the FX2 is software configurable and hence
could end up under many different VID:PID names based on what firmware is
currently loaded onto it.
"""

import os
from collections import namedtuple


PathBase = namedtuple('PathBase', ['bus', 'address'])


class Path(PathBase):

    def __new__(cls, *args, **kw):
        r = PathBase.__new__(cls, *args, **kw)
        assert os.path.exists(r.path), "%r %r" % (r.path, r)
        return r

    @property
    def path(self):
        return '/dev/bus/usb/%03i/%03i' % (self.bus, self.address)

    def __str__(self):
        return self.path

    def __cmp__(self, other):
        if isinstance(other, Path):
            return cmp(self.path, other.path)
        return cmp(self.path, other)


_DeviceBase = namedtuple(
    'DeviceBase', ['path', 'vid', 'pid', 'did', 'serialno'])


class DeviceBase(_DeviceBase):

    def __repr__(self):
        if self.serialno:
            s = repr(self.serialno)
        else:
            s = self.path
        return "%s(%04x:%04x:%s %s)" % (
            self.__class__.__name__, self.vid, self.pid, self.did, s)



#!/usr/bin/env python3
# vim: set ts=4 sw=4 et sts=4 ai:

"""
Functions for examining different file types.
"""

import struct


def assert_eq(a, b):
    assert a == b, "'%s' (%r) != '%s' (%r)" % (a, a, b, b)


class XilinxBitFile(object):
    """
    This page describes the format
    http://www.fpga-faq.com/FAQ_Pages/0026_Tell_me_about_bit_files.htm

    Field 1
    2 bytes     length 0x0009           (big endian)
    9 bytes     0f f0 0f f0 0f f0 0f f0 00
    2 bytes     00 01

    Field 3
    1 byte      key 0x61                (The letter "a")
    2 bytes     length 0x000a           (value depends on file name length)
    10 bytes    string design name "xform.ncd" (including a trailing 0x00)

    Field 4
    1 byte      key 0x62                (The letter "b")
    2 bytes     length 0x000c           (value depends on part name length)
    12 bytes    string part name "v1000efg860" (including a trailing 0x00)

    Field 4
    1 byte      key 0x63                (The letter "c")
    2 bytes     length 0x000b
    11 bytes    string date "2001/08/10" (including a trailing 0x00)

    Field 5
    1 byte      key 0x64                (The letter "d")
    2 bytes     length 0x0009
    9 bytes     string time "06:55:04"  (including a trailing 0x00)

    Field 6
    1 byte      key 0x65                (The letter "e")
    4 bytes     length 0x000c9090       (value depends on device type,
                                         and maybe design details)
    """

    header = struct.Struct(
        ">"   # big endian
        "H"   # h1, beshort == 0x0009
        "9s"  # 0f f0 0f f0 0f f0 0f f0 00
        "2s"  # h4, null byte
    )

    sfmt = struct.Struct(">ch")

    @classmethod
    def unpack_key(cls, f):
        d = f.read(cls.sfmt.size)
        key, slen = cls.sfmt.unpack(d)
        s = f.read(slen - 1)
        null = f.read(1)
        assert_eq(null, b'\x00')
        return key.decode('ascii'), s.decode('ascii')

    def __init__(self, filename):
        try:
            assert filename.endswith('.bit'), "Filename should end in .bit"
            f = open(filename, 'rb')

            # Read the header
            data = f.read(self.header.size)
            (h1, h2, h3) = self.header.unpack_from(data)
            assert_eq(h1, 0x0009)
            assert_eq(h2, b'\x0f\xf0\x0f\xf0\x0f\xf0\x0f\xf0\x00')
            assert_eq(h3, b'\x00\x01')

            self.ncdname = None
            self.part = None
            self.date = None

            while True:
                key, value = self.unpack_key(f)
                if key == 'a':
                    self.ncdname = value
                elif key == 'b':  # Part type
                    self.part = value
                elif key == 'c':  # Build date
                    self.date = value
                elif key == 'd':  # Build time
                    self.date += " " + value
                    break
                elif key == 'e':  # ????
                    break

            assert self.ncdname
            assert self.part
            assert self.date
        except AssertionError as e:
            raise TypeError(e)

    def __str__(self):
        return "{}(ncdname={!r}, part={!r}, date={!r})".format(
            self.__class__.__name__, self.ncdname, self.part, self.date)


class XilinxBinFile(object):
    HEADER = b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xaa\x99Uf0\xa1\x00\x07'  # noqa

    def __init__(self, filename):
        if not filename.endswith('.bin'):
            raise TypeError("Filename should end in .bin")

        hdr = open(filename, 'rb').read(len(self.HEADER))
        if hdr != self.HEADER:
            raise TypeError("File doesn't start with required header.")


if __name__ == "__main__":
    import sys
    fname = sys.argv[1]
    if fname.endswith('.bin'):
        print(XilinxBinFile(fname))
    elif fname.endswith('.bit'):
        print(XilinxBitFile(fname))
    else:
        sys.exit(1)



#!/usr/bin/env python3
# vim: set ts=4 sw=4 et sts=4 ai:

"""
Tool to figure find the USB device that a board is available at.

This is a complicated process as the FX2 is software configurable and hence
could end up under many different VID:PID names based on what firmware is
currently loaded onto it.
"""

import logging
import os
import os.path
import sys
import time
import subprocess

import argparse

from collections import namedtuple

from . import boards


def args_parser(board, mode):
    """Create an parser for the arguments."""
    parser = argparse.ArgumentParser(description=__doc__)

    # , aliases=['--debug', '-d'])
    parser.add_argument(
        '--verbose',
        '-v',
        action='count',
        default=0,
        help='Output more information.')

    parser.add_argument(
        '--by-type',
        choices=boards.BOARD_TYPES,
        help='Find board with a given type.')

    parser.add_argument(
        '--by-mac',
        help='Find board with the given MAC address.')
    parser.add_argument(
        '--by-dna',
        help='Find board with the given Device DNA.')
    parser.add_argument(
        '--by-position',
        help="""\
Find board using a given position in the USB structure.

Example:
 1-2.3 - Bus 1, Port 2 (which is a hub), Port 3
 5-6.7.8 - Bus 5, Port 2 (which is a hub), Port 7 (which is a hub), Port 8 (which is a hub)

While this *should* be static across reboots, but sadly on some machines it isn't :(
""")  # noqa
    parser.add_argument(
        '--by-mode',
        help=argparse.SUPPRESS)  # help='Find board in a given mode.', )

    parser.add_argument(
        '--all',
        help="""\
Do operation on all boards, otherwise will error if multiple boards are found.
""")

    parser.add_argument(
        '--get-usbfs',
        action='store_true',
        help='Return the /dev/bus/usb path for a device.')
    parser.add_argument(
        '--get-sysfs',
        action='store_true',
        help='Return the /sys/bus/usb/devices path for a device.')
    parser.add_argument(
        '--get-state',
        action='store_true',
        help='Return the state the device is in. Possible states are: %r' %
        boards.BOARD_STATES)
    parser.add_argument(
        '--get-video-device',
        action='store_true',
        help='Get the primary video device path.')
    parser.add_argument(
        '--get-serial-device',
        action='store_true',
        help='Get the serial device path.')

    parser.add_argument(
        '--prefer-hardware-serial',
        help='Prefer the hardware serial port on the Atlys board.')

    parser.add_argument(
        '--mode',
        help='Switch mode to given state.',
        choices=boards.BOARD_STATES)
    # FPGA
    parser.add_argument(
        '--load-gateware',
        help='Load gateware onto the FPGA.')
    parser.add_argument(
        '--flash-gateware',
        help='Flash gateware onto the SPI flash which the FPGA boots from.')
    # Cypress FX2
    parser.add_argument(
        '--load-fx2-firmware',
        help='Load firmware file onto the Cypress FX2.')
    parser.add_argument(
        '--flash-fx2-eeprom',
        help='Flash the FX2 eeprom with data.')
    #
    parser.add_argument(
        '--load-lm32-firmware',
        help="""\
Load firmware file onto the lm32 Soft-Core running inside the FPGA.
""")

    parser.add_argument(
        '--timeout',
        help='How long to wait in seconds before giving up.',
        type=float)

    return parser


def find_boards(args):
    all_boards = boards.find_boards(verbose=args.verbose)

    # Filter out the boards we don't care about
    filtered_boards = []
    for board in all_boards:
        if args.verbose > 0:
            sys.stderr.write("%s in '%s' mode at %s\n" % (
                boards.BOARD_NAMES[board.type],
                board.state,
                board.dev.path,
            ))
            for sp in board.dev.syspaths:
                sys.stderr.write(" %s\n" % (sp,))

            if board.dev.inuse():
                sys.stderr.write(
                    " Board is currently used by drivers %s\n" %
                    (board.dev.drivers(),))

            if board.tty():
                sys.stderr.write(" Serial port at %s\n" %
                                 ", ".join(board.tty()))

        if args.by_type and args.by_type != board.type:
            if args.verbose > 0:
                sys.stderr.write(" Ignore as not type %s\n" % (args.by_type,))
            continue

        filtered_boards.append(board)

    return filtered_boards


def main():
    # Parse the command line name
    cmd = os.path.basename(sys.argv[0])
    if cmd.endswith('.py'):
        cmd = cmd.rsplit('.', 1)[0]

    board, mode = cmd.split('-', 1)
    boards.assert_in(board, boards.BOARD_TYPES + ['hdmi2usb'])
    POSSIBLE_MODES = ['find-board', 'mode-switch', 'manage-firmware']
    boards.assert_in(mode, POSSIBLE_MODES)

    args = args_parser(mode, board).parse_args()

    if board != "hdmi2usb":
        args.by_type = board
    if args.by_type:
        boards.assert_in(args.by_type, boards.BOARD_TYPES)

    found_boards = find_boards(args)
    if not args.all:
        assert len(found_boards) == 1, found_boards

    MYDIR = os.path.dirname(os.path.abspath(__file__))
    if args.verbose:
        sys.stderr.write("My root dir: %s\n" % MYDIR)

    if mode == 'mode-switch':
        assert len(found_boards) == 1
        for board in found_boards:
            if (args.load_gateware or args.flash_gateware) and not args.mode:
                args.mode = 'jtag'

            # Switch modes
            if args.mode:
                newmode = args.mode
                if newmode == "jtag":
                    # Works on all boards
                    pass

                elif newmode in ("serial", "eeprom"):
                    assert board.type == "opsis", (
                        "{} mode only valid on the opsis.".format(newmode))

                elif newmode == "operational":
                    raise NotImplemented("Not yet finished...")
                else:
                    raise NotImplemented("Unknown mode {}".format(newmode))

                if board.state != newmode:
                    if args.verbose:
                        sys.stderr.write("Going from %s to %s\n" %
                                         (board.state, newmode))

                    old_board = board
                    boards.load_fx2(old_board, mode=newmode,
                                    verbose=args.verbose)

                    starttime = time.time()
                    while True:
                        found_boards = find_boards(args)

                        found_board = None
                        for new_board in found_boards:
                            if new_board.type == old_board.type:
                                if new_board.state == old_board.state:
                                    continue
                                assert new_board.state == newmode
                                found_board = new_board
                                break
                        else:
                            time.sleep(1)

                        if found_board:
                            board = found_board
                            break

                        if (args.timeout and starttime -
                                time.time() > args.timeout):
                            raise SystemError("Timeout!")

                    if args.verbose:
                        sys.stderr.write("Board was %r\n" % (old_board,))
                        sys.stderr.write("Board now %r\n" % (board,))
                else:
                    if args.verbose:
                        sys.stderr.write(
                            "Board already in required mode (%s)\n" %
                            (board.state,))

            # Load firmware onto the fx2
            if args.load_fx2_firmware:
                boards.load_fx2(board, filename=args.load_fx2_firmware,
                                verbose=args.verbose)

            # Load gateware onto the FPGA
            elif args.load_gateware:
                boards.load_fpga(board, args.load_gateware,
                                 verbose=args.verbose)

            elif args.flash_gateware:
                boards.flash_fpga(board, args.flash_gateware,
                                  verbose=args.verbose)

            # Load firmware onto the lm32
            elif args.load_lm32_firmware:
                if board.type == "opsis":
                    assert board.state == "serial"
                assert board.tty
                print("flterm something....")
                raise NotImplemented("Not yet finished...")

        found_boards = find_boards(args)

    for board in found_boards:
        if not (args.get_usbfs or
                args.get_sysfs or
                args.get_video_device or
                args.get_serial_device):
            print("Found %s boards." % len(found_boards))
            break

        if args.get_usbfs:
            print(board.dev.path)

        if args.get_sysfs:
            print("\n".join(board.dev.syspaths))

        if args.get_state:
            print(board.state)

        if args.get_video_device:
            assert board.state == "operational"
            print("???")

        if args.get_serial_device:
            print(board.tty()[0])



#!/usr/bin/env python3
# vim: set ts=4 sw=4 et sts=4 ai:

"""
Tests which show the libusb and lsusb implementations work the same way.
"""

from . import libusb
from . import lsusb


def test_libusb_and_lsusb_equal():
    libusb_devices = libusb.find_usb_devices()
    lsusb_devices = lsusb.find_usb_devices()
    for libobj, lsobj in zip(sorted(libusb_devices), sorted(lsusb_devices)):
        # print("%s -- lib: %-40s ls: %-40s -- %-40s  drivers: %s" % (libobj.path, libobj, lsobj, find_sys(libobj.path)[0], lsobj.drivers()))  # noqa
        print("%s -- lib: %-60s ls: %-60s -- %-40s  drivers: %s" %
              (libobj.path, libobj, lsobj, libobj.path, lsobj.drivers()))
        assert libobj.vid == lsobj.vid, "vid: %r == %r" % (
            libobj.vid, lsobj.vid)
        assert libobj.pid == lsobj.pid, "pid: %r == %r" % (
            libobj.pid, lsobj.pid)
        assert libobj.path == lsobj.path, "path: %r == %r" % (
            libobj.path, lsobj.path)

        try:
            assert libobj.did == lsobj.did, "did: %r == %r" % (
                libobj.did, lsobj.did)
        except AssertionError as e:
            print(e)

        try:
            assert libobj.serialno == lsobj.serialno, "serialno: %r == %r" % (
                libobj.serialno, lsobj.serialno)
        except AssertionError as e:
            print(e)

        lsobj_inuse = lsobj.inuse()
        libobj_inuse = libobj.inuse()
        if libobj_inuse is not None:
            assert libobj_inuse == lsobj_inuse, "inuse: %r == %r" % (
                libobj_inuse, lsobj_inuse)

test_libusb_and_lsusb_equal()



#!/usr/bin/env python
# vim: set ts=4 sw=4 et sts=4 ai:

"""
Tool to figure find the USB device that a board is available at.

This is a complicated process as the FX2 is software configurable and hence
could end up under many different VID:PID names based on what firmware is
currently loaded onto it.
"""

import logging
import os
import os.path
import sys
import time
import subprocess

from collections import namedtuple

from . import lsusb as usbapi
from . import files


def assert_in(needle, haystack):
    assert needle in haystack, "%r not in %r" % (needle, haystack)


__filepath__ = os.path.dirname(__file__)
FIRMWARE_DIR = os.path.abspath(os.path.realpath(
    os.path.join(__filepath__, '..', 'firmware')))
assert os.path.exists(FIRMWARE_DIR)


def firmware_path(filepath):
    locations = ['']
    locations.append(os.getcwd())
    locations.append(FIRMWARE_DIR)

    for loc in locations:
        fullname = os.path.join(loc, filepath)
        fullname = os.path.abspath(os.path.realpath(fullname))
        if os.path.exists(fullname):
            return fullname

    assert False, "{} not found in {}".format(filepath, locations)


BOARD_TYPES = [
    'opsis',
    'atlys',
]
BOARD_NAMES = {
    'atlys': "Digilent Atlys",
    'opsis': "Numato Opsis",
}
BOARD_STATES = [
    'unconfigured',
    'jtag',
    'serial',
    'eeprom',
    'operational',
]
BOARD_FPGA = {
    'atlys': "6slx45csg324",
    'opsis': "6slx45tfgg484",
}

USBJTAG_MAPPING = {
    'hw_nexys': 'atlys',
    'hw_opsis': 'opsis',
}
USBJTAG_RMAPPING = {v: k for k, v in USBJTAG_MAPPING.items()}

OPENOCD_MAPPING = {
    'atlys': "board/digilent_atlys.cfg",
    'opsis': "board/numato_opsis.cfg",
}
OPENOCD_FLASHPROXY = {
    'opsis': firmware_path('spartan6/opsis/bscan_spi_xc6slx45t.bit'),
    'atlys': firmware_path('spartan6/atlys/bscan_spi_xc6slx45.bit'),
}

FX2_MODE_MAPPING = {
    'jtag': 'ixo-usb-jtag.hex',
    'serial': 'usb-uart.ihx',
    'eeprom': 'eeprom.ihx',
}


BoardBase = namedtuple("Board", ["dev", "type", "state"])


class Board(BoardBase):

    def tty(self):
        return self.dev.tty()


def load_fx2(board, mode=None, filename=None, verbose=False):
    if mode is not None:
        assert filename is None
        filename = firmware_path(
            'fx2/{}/{}'.format(board.type, FX2_MODE_MAPPING[mode]))

    if board.dev.inuse():
        if verbose:
            sys.stderr.write("Detaching drivers from board.\n")
        board.dev.detach()

    filepath = firmware_path(filename)
    assert os.path.exists(filepath), filepath

    sys.stderr.write("Using FX2 firmware %s\n" % filename)

    cmdline = "fxload -t fx2lp".split()
    cmdline += ["-D", str(board.dev.path)]
    cmdline += ["-I", filepath]
    if verbose:
        cmdline += ["-v", ]

    if verbose:
        sys.stderr.write("Running %r\n" % " ".join(cmdline))

    env = os.environ.copy()
    env['PATH'] = env['PATH'] + ':/usr/sbin:/sbin'

    try:
        output = subprocess.check_output(cmdline, stderr=subprocess.STDOUT,
                                         env=env)
    except subprocess.CalledProcessError as e:
        if b"can't modify CPUCS: Protocol error\n" not in e.output:
            print(e.output)
            raise


def flash_fx2(board, filename, verbose=False):
    assert board.state == "eeprom", board
    assert not board.dev.inuse()

    assert board.type == "opsis", (
        "Only support flashing the Opsis for now (not %s)." % board.type)


def load_fpga(board, filename, verbose=False):
    assert board.state == "jtag", board
    assert not board.dev.inuse()
    assert board.type in OPENOCD_MAPPING

    filepath = firmware_path(filename)
    assert os.path.exists(filepath), filepath
    assert filename.endswith(".bit"), "Loading requires a .bit file"
    xfile = files.XilinxBitFile(filepath)
    assert xfile.part == BOARD_FPGA[board.type], (
        "Bit file must be for {} (not {})".format(
            BOARD_FPGA[board.type], xfile.part))

    script = ["init"]
    if verbose:
        script += ["xc6s_print_dna xc6s.tap"]

    script += ["pld load 0 {}".format(filepath)]
    script += ["exit"]

    cmdline = ["openocd"]
    cmdline += ["-f", OPENOCD_MAPPING[board.type]]
    cmdline += ["-c", "; ".join(script)]

    if verbose == 0:
        subprocess.check_output(cmdline, stderr=subprocess.STDOUT)
    else:
        if verbose > 1:
            cmdline += ["--debug={}".format(verbose - 2)]
        sys.stderr.write("Running %r\n" % cmdline)
        subprocess.check_call(cmdline)


def flash_fpga(board, filename, verbose=False):
    assert board.state == "jtag", board
    assert not board.dev.inuse()
    assert board.type in OPENOCD_MAPPING

    filepath = firmware_path(filename)
    assert os.path.exists(filepath), filepath
    assert filename.endswith(".bin"), "Flashing requires a .bin file"
    xfile = files.XilinxBinFile(filepath)

    assert board.type in OPENOCD_FLASHPROXY
    proxypath = os.path.abspath(OPENOCD_FLASHPROXY[board.type])
    assert os.path.exists(proxypath), proxypath

    script = ["init"]
    if verbose:
        script += ["xc6s_print_dna xc6s.tap"]

    script += ["jtagspi_init 0 {}".format(proxypath)]

    if verbose > 1:
        script += ["flash banks"]
        script += ["flash list"]
    if verbose > 2:
        script += ["flash info 0"]

    # script += ["flash read_bank 0 backup.bit 0 0x01000000"]

    script += [
        "jtagspi_program {} 0x{:x}".format(filepath, 0),
        "exit"
    ]

    cmdline = ["openocd"]

    cmdline += ["-f", OPENOCD_MAPPING[board.type]]
    cmdline += ["-c", "; ".join(script)]

    if verbose == 0:
        subprocess.check_output(cmdline, stderr=subprocess.STDOUT)
    else:
        if verbose > 1:
            cmdline += ["--debug={}".format(verbose - 2)]
        sys.stderr.write("Running %r\n" % cmdline)
        subprocess.check_call(cmdline)


def find_boards(prefer_hardware_serial=True, verbose=False):
    all_boards = []
    exart_uarts = []
    for device in usbapi.find_usb_devices():
        if False:
            pass

        # Digilent Atlys
        # --------------------------
        # Digilent Atlys board with stock "Adept" firmware
        # Bus 003 Device 019: ID 1443:0007 Digilent Development board JTAG
        if device.vid == 0x1443 and device.pid == 0x0007:
            all_boards.append(
                Board(dev=device, type="atlys", state="unconfigured"))

        # Digilent Atlys board unconfigured mode with Openmoko ID
        # Bus 003 Device 019: ID 1d50:60b5
        elif device.vid == 0x1d50 and device.pid == 0x60b5:
            all_boards.append(
                Board(dev=device, type="atlys", state="unconfigured"))

        # Digilent Atlys board JTAG/firmware upgrade mode with Openmoko ID
        # Bus 003 Device 019: ID 1d50:60b6
        elif device.vid == 0x1d50 and device.pid == 0x60b6:
            all_boards.append(
                Board(dev=device, type="atlys", state="jtag"))

        # Digilent Atlys board JTAG/firmware upgrade mode with Openmoko ID
        # Bus 003 Device 019: ID 1d50:60b7
        elif device.vid == 0x1d50 and device.pid == 0x60b7:
            all_boards.append(
                Board(dev=device, type="atlys", state="operational"))

        elif device.vid == 0x04e2 and device.pid == 0x1410:
            exart_uarts.append(device)

        # Numato Opsis
        # --------------------------
        # The Numato Opsis will boot in the following mode when the EEPROM is
        # not set up correctly.
        # http://opsis.hdmi2usb.tv/getting-started/usb-ids.html#failsafe-mode
        # Bus 003 Device 091: ID 04b4:8613 Cypress Semiconductor Corp.
        # CY7C68013 EZ-USB FX2 USB 2.0 Development Kit
        elif device.vid == 0x04b4 and device.pid == 0x8613:
            all_boards.append(
                Board(dev=device, type="opsis", state="unconfigured"))

        # The preproduction Numato Opsis shipped to Champions will boot into
        # this mode by default.
        # The production Numato Opsis will fallback to booting in the following
        # mode when the FPGA doesn't have EEPROM emulation working.
        # http://opsis.hdmi2usb.tv/getting-started/usb-ids.html#unconfigured-mode
        # Bus 003 Device 091: ID 2a19:5440 Numato Opsis (Unconfigured Mode)
        elif device.vid == 0x2A19 and device.pid == 0x5440:
            all_boards.append(
                Board(dev=device, type="opsis", state="unconfigured"))

        # The production Numato Opsis will boot in this mode when SW1 is held
        # during boot, or when held for 5 seconds with correctly configured
        # FPGA gateware.
        # http://opsis.hdmi2usb.tv/getting-started/usb-ids.html#usb-jtag-and-usb-uart-mode
        # Bus 003 Device 091: ID 2a19:5441 Numato Opsis (JTAG and USB Mode)
        elif device.vid == 0x2A19 and device.pid == 0x5441:
            if device.did == '0001':
                all_boards.append(
                    Board(dev=device, type="opsis", state="jtag"))
            elif device.did == '0002':
                all_boards.append(
                    Board(dev=device, type="opsis", state="eeprom"))
            elif device.did == '0003':
                all_boards.append(
                    Board(dev=device, type="opsis", state="serial"))
            else:
                assert False, "Unknown mode: %s" % device.did

        # The production Numato Opsis will boot in this mode by default.
        # http://opsis.hdmi2usb.tv/getting-started/usb-ids.html#hdmi2usb.tv-mode
        # Bus 003 Device 091: ID 2a19:5441 Numato Opsis (HDMI2USB.tv mode)
        elif device.vid == 0x2A19 and device.pid == 0x5442:
            all_boards.append(
                Board(dev=device, type="opsis", state="operational"))

        # ixo-usb-jtag
        # --------------------------
        # Boards loaded with the ixo-usb-jtag firmware from mithro's repo
        # https://github.com/mithro/ixo-usb-jtag
        # Bus 003 Device 090: ID 16c0:06ad Van Ooijen Technische Informatica
        elif device.vid == 0x16c0 and device.pid == 0x06ad:
            if device.did in ('0001', '0004'):
                if device.serialno not in USBJTAG_MAPPING:
                    logging.warn("Unknown usb-jtag device! %r (%s)",
                                 device.serialno, device)
                    continue
                all_boards.append(Board(dev=device, type=USBJTAG_MAPPING[
                                  device.serialno], state="jtag"))
            elif device.did == 'ff00':
                all_boards.append(
                    Board(dev=device, type='opsis', state="jtag"))
            else:
                logging.warn(
                    "Unknown usb-jtag device version! %r (%s)",
                    device.did,
                    device)
                continue

    # FIXME: This is a horrible hack!?@
    # Patch the Atlys board so the exar_uart is associated with it.
    if exart_uarts:
        atlys_boards = [b for b in all_boards if b.type == "atlys"]
        if verbose:
            sys.stderr.write(
                " Found exart-uarts at %s associating with Atlys at %s\n" %
                (exart_uarts, atlys_boards))
        assert len(exart_uarts) == len(atlys_boards)
        assert len(atlys_boards) == 1

        def extra_tty(
                uart=exart_uarts[0],
                board=atlys_boards[0],
                prefer=prefer_hardware_serial):
            if prefer:
                return uart.tty() + board.dev.tty()
            else:
                return board.dev.tty() + uart.tty()

        atlys_boards[0].tty = extra_tty

    return all_boards



#!/usr/bin/env python3
# vim: set ts=4 sw=4 et sts=4 ai:

from setuptools import find_packages
from setuptools import setup
import os
import sys

import versioneer


if sys.version_info[:3] < (3, 3):
    raise SystemExit("You need Python 3.3+")


if os.environ.get('PYBUILD_NAME'):
    setup_requires = []
else:
    setup_requires = ['setuptools-pep8']


setup(
    name="hdmi2usb.modeswitch",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description=(
        "Module and command line tool for control the mode of HDMI2USB "
        "devices."
    ),
    long_description=open("README.md").read(),
    author="TimVideos' HDMI2USB project",
    author_email="hdmi2usb@googlegroups.com",
    url="https://hdmi2usb.tv",
    download_url="https://github.com/timvideos/HDMI2USB-mode-switch",
    license="Apache 2.0",
    platforms=["Any"],
    keywords="HDL ASIC FPGA hardware design",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)",  # noqa
    ],
    packages=find_packages(),
# FIXME: Populate this with information?
#    package_data={},
#    data_files=[
#        ('/lib/firmware/hdmi2usb', os.walk('hdmi2usb/firmware/'),
#        ('/usr/bin/', build?(unbind-helper)),
#        ('/etc/udev/rules.d/', os.listdir(udev,
#            "*-hdmi2usb-*.rules", "hdmi2usb-*.sh"),
#    ],
    setup_requires=setup_requires,
    include_package_data=True,
    entry_points={
        "console_scripts": [x+"=hdmi2usb.modeswitch.cli:main" for x in (
            "hdmi2usb-find-board", "hdmi2usb-mode-switch",
            "hdmi2usb-manage-firmware",
            "opsis-find-board", "opsis-mode-switch", "opsis-manage-firmware",
            "atlys-find-board", "atlys-mode-switch", "atlys-manage-firmware",
        )],
    },
)



#!/usr/bin/env python3
# vim: set ts=4 sw=4 et sts=4 ai:

"""
Functions needed by hdmi2usb-mode-switch implemented using lsusb and other
Linux command line tools.

This will only run on Linux.
"""

import logging
import os
import re
import subprocess

from .base import *

# Try and find unbind-helper


def find_unbind_helper():
    callpaths = [
        os.path.join(os.path.dirname(__file__), "..",
                     "..", "bin", "unbind-helper"),
        "unbind-helper",
    ]

    for path in callpaths:
        pathret = subprocess.call(
            path,
            shell=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL)
        if pathret == 255:
            return path

    logging.warning("unbind-helper not found, will have to run as root!")
    return None

unbind_helper = find_unbind_helper()


SYS_ROOT = '/sys/bus/usb/devices'


def get_path_from_sysdir(dirpath):
    buspath = os.path.join(dirpath, 'busnum')
    devpath = os.path.join(dirpath, 'devnum')
    if not os.path.exists(buspath):
        logging.info("Skipping %s (no busnum)", buspath)
        return None
    if not os.path.exists(devpath):
        logging.info("Skipping %s (no devnum)", devpath)
        return None

    busnum = int(open(buspath, 'r').read().strip())
    devnum = int(open(devpath, 'r').read().strip())

    return Path(bus=busnum, address=devnum)


def create_sys_mapping():
    # 1-1.3.1  --> (Device)    bus-port.port.port
    # 1-0:1.0  --> (Interface) bus-port.port.port:config.interface
    # usb1     --> bus<number>

    devices = {}
    interfaces = {}

    dirs = list(sorted(os.listdir(SYS_ROOT)))
    for dirname in dirs:
        dirpath = os.path.join(SYS_ROOT, dirname)
        if ":" in dirname:
            continue
        path = get_path_from_sysdir(dirpath)
        assert path
        devices[dirpath] = path
        assert path not in interfaces
        interfaces[path] = [dirpath]

    for dirname in dirs:
        dirpath = os.path.join(SYS_ROOT, dirname)
        if ":" not in dirname:
            continue

        device, interface = dirname.split(':')
        if device.endswith('-0'):
            device = "usb%s" % (device[:-2])

        devpath = os.path.join(SYS_ROOT, device)
        assert os.path.exists(devpath)
        assert devpath in devices

        interfaces[devices[devpath]].append(dirpath)

    return interfaces


FIND_SYS_CACHE = {}


def find_sys(path, mapping=FIND_SYS_CACHE):
    if not mapping:
        mapping.update(create_sys_mapping())
    return mapping[path]


class LsusbDevice(DeviceBase):

    def __new__(cls, *args, **kw):
        syspaths = sorted(find_sys(kw['path']))

        # Get the did/serialno number from sysfs
        did = None
        serialno = None

        for syspath in syspaths:
            didpath = os.path.join(syspath, "bcdDevice")
            if os.path.exists(didpath):
                newdid = open(didpath, "r").read().strip()
                assert did is None or did == newdid, (did, newdid)
                did = newdid

            serialnopath = os.path.join(syspath, "serial")
            if os.path.exists(serialnopath):
                newserialno = open(serialnopath, "r").read().strip()
                assert serialno is None or serialno == newserialno, (
                    serialno, newserialno)
                serialno = newserialno

        d = DeviceBase.__new__(cls, *args, did=did, serialno=serialno, **kw)
        d.syspaths = syspaths
        return d

    def inuse(self):
        return bool(self.drivers())

    def drivers(self):
        drivers = {}
        for path in self.syspaths[1:]:
            driver_path = os.path.join(path, "driver")
            if os.path.exists(driver_path):
                drivers[path] = os.readlink(driver_path)
        return tuple(set(d.split('/')[-1] for d in drivers.values()))

    def detach(self):
        for path in self.syspaths[1:]:
            driver_path = os.path.join(path, "driver")
            if os.path.exists(driver_path):
                unbind_path = os.path.join(driver_path, "unbind")
                assert os.path.exists(unbind_path), unbind_path
                interface = os.path.split(path)[-1]
                try:
                    open(unbind_path, "w").write(interface)
                except PermissionError:
                    if not unbind_helper:
                        raise
                    subprocess.check_call("%s '%s' '%s'" % (
                        unbind_helper, unbind_path, interface), shell=True)

    def tty(self):
        ttys = []
        for path in self.syspaths:
            tty_path = os.path.join(path, "tty")
            if os.path.exists(tty_path):
                names = list(os.listdir(tty_path))
                assert len(names) == 1
                ttys.append('/dev/' + names[0])
        return ttys


Device = LsusbDevice


def find_usb_devices():
    FIND_SYS_CACHE.clear()

    # 'Bus 002 Device 002: ID 8087:0024 Intel Corp. Integrated Rate Matching Hub'  # noqa
    lsusb_device_regex = re.compile(
        "Bus (?P<bus>[0-9]+) Device (?P<address>[0-9]+):"
        " ID (?P<vid>[0-9a-f]+):(?P<pid>[0-9a-f]+)")

    devobjs = []
    output = subprocess.check_output('lsusb')
    for line in output.splitlines():
        line = line.decode('utf-8')
        bits = lsusb_device_regex.match(line)
        assert bits, repr(line)

        vid = int(bits.group('vid'), base=16)
        pid = int(bits.group('pid'), base=16)
        bus = int(bits.group('bus'), base=10)
        address = int(bits.group('address'), base=10)
        devobjs.append(LsusbDevice(
            vid=vid, pid=pid, path=Path(bus=bus, address=address),
        ))

    return devobjs
